CREATE OR REPLACE PACKAGE BODY xmlq AS

  msgProp  dbms_aq.message_properties_t;

  ----------------------------------------------------------------------
  -- Enqueue an XML document to the (raw-payload) 'queueName' queue.
  -- Raw-payload queues have a message-size limit of 32767 bytes.
  ----------------------------------------------------------------------
  PROCEDURE enqueue( xmldoc xmldom.DOMDocument, queueName VARCHAR2 ) IS
   enqOpt dbms_aq.enqueue_options_t;
   msgHdl RAW(16);
  BEGIN
    dbms_aq.enqueue(queue_name => queueName,
               enqueue_options => enqOpt,
            message_properties => msgProp,
                       payload => utl_raw.cast_to_raw(xpath.extract(xmldoc)),
                         msgid => msgHdl);
    COMMIT;
  END;

  ----------------------------------------------------------------------
  -- Dequeue an XML document from the (raw-payload) 'queueName' queue.
  --
  -- If the 'wait' parameter is TRUE (the default) the function blocks
  -- until a message is available on the queue. If 'wait' is false,
  -- either an XML document is returned, or the 'empty_queue' exception
  -- is thrown.
  ----------------------------------------------------------------------
  FUNCTION  dequeue( queueName VARCHAR2, wait BOOLEAN := TRUE )
    RETURN  xmldom.DOMDocument IS
    deqOpt  dbms_aq.dequeue_options_t;
    retval  xmldom.DOMDocument;
    msgHdl  RAW(16);
    message RAW(32767);
  BEGIN
    IF NOT wait THEN
      deqOpt.wait := dbms_aq.NO_WAIT;
    END IF;
    dbms_aq.dequeue(queue_name => queueName,
               dequeue_options => deqOpt,
            message_properties => msgProp,
                       payload => message,
                         msgid => msgHdl);
    COMMIT;
    RETURN xml.parse(utl_raw.cast_to_varchar2(message));
  END;

END;
