CREATE OR REPLACE PACKAGE BODY ForumSearch IS
  FUNCTION Query(forumid   NUMBER   := -1,
                 daysAgo   NUMBER   := 0,
                 searchIn  VARCHAR2 := 'S',
                 searchFor VARCHAR2 := NULL) RETURN VARCHAR2 IS
    query  VARCHAR2(2000);
    noCriteriaSupplied  BOOLEAN := LTRIM(RTRIM(searchFor)) IS NULL;
    searchEntireMessage BOOLEAN := UPPER(searchIn)='E';
    restrictToForum     BOOLEAN := forumid > 0;
    restrictByDate      BOOLEAN := daysAgo < 0;
  BEGIN
    query :=
    'select distinct
            t.id              as h_id,
            t.title           as "Topic",
            t.userid          as "Started_By",
            t.postings-1      as "Replies",
            to_char(t.lastpost,''dd Mon YYYY, hh24:mi'') as "Last_Post",
            f.name            as "Forum_Name"
       from forum_topic t,
            forum_topic_posting p,
            forum f
      where t.id = p.topicid
        and t.forumid = f.id';
    IF searchEntireMessage THEN
      query := query ||' AND ( CONTAINS(p.posting, '''||searchFor||''')>0 '||
                       ' OR UPPER(t.title) LIKE UPPER(''%'||searchFor||'%''))';
    ELSE
      query := query || ' AND UPPER(t.title) LIKE UPPER(''%'||searchFor||'%'')';
    END IF;
    IF restrictToForum THEN
      query := query ||' AND t.forumid = '||forumid;
    END IF;
    IF restrictByDate THEN
      query := query ||' AND p.posted > SYSDATE + '||daysAgo;
    END IF;
    IF noCriteriaSupplied THEN
      query := query ||' AND 1=2';
    END IF;
    RETURN query;
  END;

  FUNCTION Hits(forumid   NUMBER   := -1,
                daysAgo   NUMBER   := 0,
                searchIn  VARCHAR2 := 'S',
                searchFor VARCHAR2 := NULL) RETURN NUMBER IS
    the_cursor ref_cursor;
    the_count  NUMBER := 0;
  BEGIN
    OPEN the_cursor FOR 'select count(1) from ('||
                        Query(forumid,daysAgo,searchIn,searchFor)||
                        ')';
    FETCH the_cursor INTO the_count;
    CLOSE the_cursor;
    RETURN the_count;
  END;

  FUNCTION Find(forumid   NUMBER   := -1,
                daysAgo   NUMBER   := 0,
                searchIn  VARCHAR2 := 'S',
                searchFor VARCHAR2 := NULL) RETURN ref_cursor IS
    the_cursor ref_cursor;
    my_sal NUMBER := 1;

 BEGIN
   OPEN the_cursor FOR Query(forumid,daysAgo,searchIn,searchFor)||
                       ' order by 5 desc';
   RETURN the_cursor;
  END;
END;
