<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text" media-type="text/plain"/>
  <xsl:strip-space elements="*"/>
  <xsl:param name="table"/>
  <xsl:key name="md" match="METADATA/ROW" use="COLUMN_NAME"/>
  <xsl:template match="/">
    <xsl:text>set termout on&#xa;</xsl:text>
    <xsl:text>set feedback off&#xa;</xsl:text>
    <xsl:text>set verify off&#xa;</xsl:text>

    <xsl:text>PROMPT Dropping table </xsl:text>
    <xsl:value-of select="$table"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>DROP TABLE </xsl:text>
    <xsl:value-of select="$table"/>
    <xsl:text>;&#xa;</xsl:text>

    <xsl:text>PROMPT Creating table </xsl:text>
    <xsl:value-of select="$table"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>CREATE TABLE </xsl:text>
    <xsl:value-of select="$table"/>
    <xsl:text> (&#xa;</xsl:text>
    <xsl:call-template name="column-list">
      <xsl:with-param name="data-types" select="true()"/>
    </xsl:call-template>
    <xsl:text>);&#xa;</xsl:text>

    <xsl:text>PROMPT Inserting data into </xsl:text>
    <xsl:value-of select="$table"/>
    <xsl:text>...&#xa;</xsl:text>
    <xsl:for-each select="EXPORT/ROWSET/ROW">
      <xsl:text>INSERT INTO </xsl:text>
      <xsl:value-of select="$table"/>
      <xsl:text>(</xsl:text>
      <xsl:call-template name="column-list"/>
      <xsl:text>)&#xa;VALUES (&#xa;</xsl:text>
      <xsl:call-template name="values-list"/>
      <xsl:text>);&#xa;</xsl:text>
    </xsl:for-each>
    <xsl:text>commit;&#xa;</xsl:text>
    <xsl:text>PROMPT Done.&#xa;</xsl:text>
    <xsl:text>set feedback on&#xa;&#xa;</xsl:text>
    <xsl:text>set verify on&#xa;&#xa;</xsl:text>
  </xsl:template>
  <xsl:template name="column-list">
    <xsl:param name="data-types" select="false()"/>
    <xsl:for-each select="/EXPORT/METADATA/ROW/COLUMN_NAME">
      <xsl:value-of select="."/>
      <xsl:if test="$data-types">
        <xsl:apply-templates select="../DATA_TYPE"/>
        <xsl:text>&#xa;</xsl:text>
      </xsl:if>
      <xsl:if test="position()!=last()">,</xsl:if>
    </xsl:for-each>
  </xsl:template>
  <xsl:template name="values-list">
    <xsl:for-each select="*">
      <xsl:variable name="DT" select="key('md',name(.))/DATA_TYPE"/>
      <xsl:if test="$DT != 'NUMBER'">'</xsl:if>
      <xsl:call-template name="apos-replace">
        <xsl:with-param name="text" select="."/>
      </xsl:call-template>
      <xsl:if test="$DT != 'NUMBER'">'</xsl:if>
      <xsl:if test="position()!=last()">,&#xa;</xsl:if>
    </xsl:for-each>
  </xsl:template>
  <xsl:template name="apos-replace">
    <xsl:param name="text"/>
    <xsl:variable name="sp"><xsl:text>'</xsl:text></xsl:variable>
    <xsl:choose>
      <xsl:when test="contains($text,$sp)">
        <xsl:value-of select="substring-before($text,$sp)"/>
        <xsl:text>''</xsl:text>
        <xsl:call-template name="apos-replace">
          <xsl:with-param name="text" select="substring-after($text,$sp)"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="DATA_TYPE">
    <xsl:text> </xsl:text>
    <xsl:choose>
      <xsl:when test=".='VARCHAR2'">
        <xsl:value-of select="."/>
        <xsl:text>(</xsl:text>
        <xsl:value-of select="../DATA_LENGTH"/>
        <xsl:text>)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:stylesheet>